@@ -80,11 +80,20 @@ static void
 g_union_volume_monitor_dispose (GObject *object)
 {
   GUnionVolumeMonitor *monitor;
-  
+  GVolumeMonitor *child_monitor;
+  GList *l;
+
   monitor = G_UNION_VOLUME_MONITOR (object);
 
   g_static_rec_mutex_lock (&the_volume_monitor_mutex);
   the_volume_monitor = NULL;
+
+  for (l = monitor->monitors; l != NULL; l = l->next)
+    {
+      child_monitor = l->data;
+      g_object_run_dispose (G_OBJECT (child_monitor));
+    }
+  
   g_static_rec_mutex_unlock (&the_volume_monitor_mutex);
 
   G_OBJECT_CLASS (g_union_volume_monitor_parent_class)->dispose (object);
@@ -610,14 +619,14 @@ _g_mount_get_for_mount_path (const char *mount_path,
  *
  * If the return value is not %NULL, the caller must associate the
  * returned #GVolume object with the #GMount. This involves returning
- * it in it's g_mount_get_volume() implementation. The caller must
+ * it in its g_mount_get_volume() implementation. The caller must
  * also listen for the "removed" signal on the returned object
- * and give up it's reference when handling that signal
+ * and give up its reference when handling that signal
  * 
  * Similary, if implementing g_volume_monitor_adopt_orphan_mount(),
  * the implementor must take a reference to @mount and return it in
- * it's g_volume_get_mount() implemented. Also, the implementor must
- * listen for the "unmounted" signal on @mount and give up it's
+ * its g_volume_get_mount() implemented. Also, the implementor must
+ * listen for the "unmounted" signal on @mount and give up its
  * reference upon handling that signal.
  *
  * There are two main use cases for this function.
@@ -625,7 +634,7 @@ _g_mount_get_for_mount_path (const char *mount_path,
  * One is when implementing a user space file system driver that reads
  * blocks of a block device that is already represented by the native
  * volume monitor (for example a CD Audio file system driver). Such
- * a driver will generate it's own #GMount object that needs to be
+ * a driver will generate its own #GMount object that needs to be
  * assoicated with the #GVolume object that represents the volume.
  *
  * The other is for implementing a #GVolumeMonitor whose sole purpose
@@ -638,7 +647,8 @@ _g_mount_get_for_mount_path (const char *mount_path,
  * Deprecated: 2.20: Instead of using this function, #GVolumeMonitor
  * implementations should instead create shadow mounts with the URI of
  * the mount they intend to adopt. See the proxy volume monitor in
- * gvfs for an example of this.
+ * gvfs for an example of this. Also see g_mount_is_shadowed(),
+ * g_mount_shadow() and g_mount_unshadow() functions.
  */
 GVolume *
 g_volume_monitor_adopt_orphan_mount (GMount *mount)
